//knihovny
#include <SPI.h>
#include <SD.h>
#include <OneWire.h>
#include <DallasTemperature.h>
#define ONE_WIRE_BUS 2
OneWire oneWire(ONE_WIRE_BUS);
DallasTemperature sensors(&oneWire);
//vstupní nastavení
const int chipSelect = 10; //sběrnice spi, pin 10 zvolen jako CS

//globální proměnné
long memory[20]={0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}; //pole pamětí pro čítače, časovače, případně další
bool m_set[20]={0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}; //pro uložení log.1, je-li memory inicializována
bool sr_block[30]={0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}; //paměti pro set/reset bloky
long t=0; //pro ukladani casu
int i=0; //pro udrzeni pozice pamety
int j=0; //pro udrzeni pozice pamety
int teplota=0; //pro teplomer
int analog_read=0; //pro ulozeni analogoveho vstupu 

//běhové instrukce
File dataFile; //pro práci se souborem na SD kartě
unsigned int ins_code_line=0; //na kterém řádku v SD kartě se nachází kurzor
String ins_incoming; //prichozi instrukce
char c; //parsovani vstupnich instrukci
byte data; //parsovani vstupnich instrukci
int ins_next_param=0; //pro posouvání mezi parametry ins_number, ins_param_one a ins_param_count
int ins_number=0; //navedeni na CASE ve switch
int ins_param_one=0; //prvni parametr funkce v danem CASE ve switch
int ins_param_two=0; //druhy parametr funkce v danem CASE ve switch
long ins_param_count=0; //druhy parametr funkce v danem CASE ve switch
int v_pin=0; //uchování pinu simulovaného přes serial
int set_pin=0; //použito v zapisování instrukcí ze serialu na spi
//
bool l_signal=0; //upravuje jesi je logicky signal pri pruchodu programem 1 nebo 0
bool run_core=0; //pokud je 1 bezi vypocetni jadro, jinak probiha zpracovani dat ze sbernic
String serial_input; //pro uložení zprávy ze serialu

void software_Reset() // restartuje program od začátku, ale neresetuje periferie a registry
   {
     asm volatile ("  jmp 0");  
   }

void setup() {
  // vyber jestli je digitalni pin vstup nebo vystup
  Serial.begin(9600);
  Serial.print("Inicializace SD karty...");
  pinMode(10, OUTPUT); 
  //zkontroluje inicializaci karty
  if (!SD.begin(chipSelect)) {
    Serial.println("\nKarta nenalezena...");    
    return;
  }
  Serial.println("\nKarta inicializovana...");
  dataFile = SD.open("setup.txt", FILE_READ);
  while(dataFile.available()){
    data = dataFile.read(); //precte jeden byte a nastavi kurzor na dalsi
    c = data;               //byte na char
    if(c==','){             //pokud carka presun se na dalsi parametr 
       ins_next_param++;
     }
     else if(data<33){ //tisknutelne znaky jsou od 33 vis v asci tabulce
      Serial.println("Neplatne nastaveni v setupu.");
      break;
     }
     else if(c==';') break;
     else{            //parsovani vstupu a nastaveni vstupne/vystupnich pinu
      //if (c < '0' || c > '9') break;
      switch(ins_next_param){
        case 0:      
          set_pin *= 10;
          set_pin += (c - '0');
       break;
       case 1:
          ins_next_param=-1;
             if(c=='O'){
                pinMode(set_pin, OUTPUT);
             }
             else if (c=='I'){
                pinMode(set_pin, INPUT);
             }
       break;
      }
    }
  }
  dataFile.close();
}

void loop() {
//čtení dat za chodu
if(Serial.available() > 0){
  serial_input=Serial.readStringUntil(';');
  //nahrání programu ze sériové linky s restartem
  if(serial_input=="write"){
    dataFile.close();
    SD.remove("setup.txt");
    dataFile = SD.open("setup.txt", FILE_WRITE);
    Serial.println("Nastavujete konfiguracni data.");
    while(1){
      serial_input=Serial.readStringUntil('\n');
      if(serial_input=="end;") break;
      if(serial_input.length()>1) dataFile.println(serial_input);
      }
    dataFile.close();
    SD.remove("ins.txt");
    dataFile = SD.open("ins.txt", FILE_WRITE);
    Serial.println("Nastavujete program.");
    while(1){
      serial_input=Serial.readStringUntil('\n');
      if(serial_input=="end;") break;
      if(serial_input.length()>1) dataFile.println(serial_input);
      }
    dataFile.close();
    Serial.println("Dokonceno");
    delay(100);
    software_Reset();
    }
    else if(serial_input=="input"){
      while(1){
        serial_input=Serial.readStringUntil(';');
        for(i=0; i<serial_input.length(); i++){
          c=serial_input.charAt(i);
          if(c<'0' || c>'9') break;
          v_pin*=10;
          v_pin+=(c-'0');
        }
        if(serial_input=="end") break;
      }
    }
  }
//běžný režim
t=millis(); //ulozi aktualni cas do promene t, pri kazdem pruchodu cyklem
//ziskani instrukci z .txt
if(!dataFile.available()){
  dataFile.close(); 
  dataFile = SD.open("ins.txt", FILE_READ);
  ins_code_line=0;
  ins_next_param=0;
  }
else{
  data = dataFile.read();
  c = data;
  //rozlozeni instrukci do pozadovanych promenych
    if(c==','){
       ins_next_param++;
     }
     else if(c==';'){
      run_core=1;      
     }
     else if(c=='-'){
      ins_param_count = ins_param_count*(-1);
     }
     else if(data<33)
     {
      //zachycení nezobrazitelných znaků a nového řádku
     }
     else{
      if (c < '0' || c > '9') {
        Serial.print("Chyba!! Chyba se vyskytla na radku: ");
        Serial.println(ins_code_line);
      }
      switch(ins_next_param){
        case 0:      
          ins_number *= 10;
          ins_number += (c - '0');
        break;
        case 1:
          ins_param_one *= 10;
          ins_param_one += (c - '0');
        break;
        case 2:
          ins_param_count *= 10;
          ins_param_count += (c - '0');
        break;
        case 3:
          ins_param_two *= 10;
          ins_param_two += (c - '0');
        break;
      }
    }
}
if(run_core){
  //Serial.println(ins_param_count);
  //delay(500);
  //vlastni instrukcemi rizeny program
  switch(ins_number){
  //inicializace casovace cislo ins_param_one na hodnotu ins_param_count
    case 1:
    //Serial.println(l_signal);
    if(l_signal==1){
      i = ins_param_one;
      if(m_set[i]==0){
        memory[i]=memory[i]+t+ins_param_count;
        (m_set[i])=1;        
      }
    }
    //Serial.println(l_signal);
    break;
  //sledovani casovace jestli docasoval
    case 2: 
      i = ins_param_one;
      if((m_set[i]==1)&&((memory[i])<t)&&(l_signal==1)){
         l_signal=1; 
         //Serial.print("a");
      }
      else l_signal=0;
   // Serial.println(l_signal);
    break;
  //digitalni vstup
    case 3:    
      if(((digitalRead(ins_param_one))==1)&&(l_signal==1)){
          l_signal=1;
      }
      else l_signal=0;
    break;
  //digitalni vystup
    case 4:
    //Serial.println(l_signal);
      if(l_signal==1){
        digitalWrite(ins_param_one, HIGH);
      }
      else digitalWrite(ins_param_one, LOW);
    break;
    //zdroj signalu
    case 5:
      l_signal=1;
    break;
    //reset casovace a citace
    case 6:
      if(l_signal==1){
        i=ins_param_one;
        m_set[i]=0;
        memory[i]=0;
      }
    break;
    //pricitani k citaci po jedne
    case 7:
      i = ins_param_one;
      if(l_signal==1){
        if(m_set[i]==0){
          memory[i]++;
          m_set[i]=1; 
        } 
      }  
      else m_set[i]=0;  
    break;
    //hlidani jestli citac ins_param_one nepretekl hodnotu ins_param_count
    case 8:
      if(l_signal==1){
        i = ins_param_one;
        if(memory[i]>=ins_param_count){
          l_signal=1;
        }
        else l_signal=0;
      }
    break;
    //set bloku cislo sr_block /set priority asi...nebo co je prvni v kodu spis
    case 9:
      i=ins_param_one;
      if(l_signal==1){
        sr_block[i]=1;
      }
      if(sr_block[i]==1){
        l_signal=1;
      }
      else l_signal=0;
    break;
    //reset blok set bloku cislo sr_block
    case 10:
      i=ins_param_one;
      if(l_signal==1){
        sr_block[i]=0;
      }
    break;
    //skakaci funkce or, pokud signal 1, přeskoč 1blok, jinak signál z 0 na 1 
    case 11:
      if(l_signal==0) l_signal=1;
      else{
        data = dataFile.read();
        c = data;
        while(c!=';'){
        data = dataFile.read();
        c = data;
        }
      }
    break;
    //not
    case 12:
      l_signal=!l_signal;
    break;
    //virtualni vstup
    case 13:
      if(l_signal==1){
        if(v_pin==ins_param_one){
          l_signal=1;
        }
        else l_signal=0;
      }
      v_pin=0;
    break;
    //virtualni vystup
    case 14:
    if(l_signal==1){
      Serial.println(ins_param_one);
    }
    break;
    //mereni teploty NA PINU D2
    case 15:
    i=ins_param_two;
    if(l_signal==1){
       if(m_set[i]==0){
          memory[i]=ins_param_count;
          m_set[i]=1; 
       } 
       sensors.requestTemperatures();
       teplota = round(sensors.getTempCByIndex(0));
       if(teplota<memory[i] && ins_param_one==0){
        l_signal=1;
       }
       else if(teplota>memory[i] && ins_param_one==1){
        l_signal=1;
       }
       else if(ins_param_one==2){
        Serial.println(teplota);
       }
       else l_signal=0;
    }
    break;
    //analogovy vstup
    case 16:
    if(l_signal==1){     
      analog_read = analogRead(ins_param_one);
    }
    break;
    //ulozeni hodnoty analogoveho vstupu do pameti
    case 17:
    i = ins_param_one;
    if(l_signal==1){
      memory[i]=analog_read;
      m_set[i]=1;
    }  
    break;
    //zobrazeni slotu v pameti
    case 18:
    i=ins_param_one;
    if(l_signal==1){     
      Serial.println(memory[i]);
    }
    break;
    //pricteni jednoho slotu z pameti k druhemu
    case 19:
    i=ins_param_one;
    j=ins_param_two;
    if(l_signal==1){
      switch(ins_param_count){
        case 0:
        memory[i]=memory[i]*memory[j];
        break;
        case 1:
        memory[i]=memory[i]/memory[j];
        break;
        case 2:
        memory[i]=memory[i]+memory[j];
        break;
        case 3:
        memory[i]=memory[i]-memory[j];
        break;
        case 4:
        memory[i]=memory[i]^memory[j];
        break;
        case 5:
        memory[i]=memory[i]^(1/memory[j]);
        break;
        case 6:
        memory[i]=memory[j];
        m_set[i]=1; 
        break;
        case 7:
        if(memory[i]>memory[j]){
          l_signal=1;
        }
        else l_signal=0;
        break;
        case 8:
        if(memory[i]<memory[j]){
          l_signal=1;
        }
        else l_signal=0;
        break;
        case 9:
        if(memory[i]==memory[j]){
          l_signal=1;
        }
        else l_signal=0;
        break;
        
      }
    }
    break;
    //pricteni cisla do pameti
    case 20:
    i=ins_param_one;
    if(l_signal==1){
      if(m_set[i]==0){
        memory[i]=memory[i]+ins_param_count;
        m_set[i]=1; 
      }
    }
    else m_set[i]=0;  
    break;
    default:
      Serial.print("Tato instrukce neni podporovana: ");
      Serial.println(ins_number);
      Serial.print("Chyba se vyskytla na radku: ");
      Serial.println(ins_code_line);
    break;
    }
  //nulování jádrových instrukcí, aby se k nim nepřičítaly hodnoty z práve uběhlého chodu programu
  ins_next_param=0;
  ins_code_line++;
  ins_number=0;
  ins_param_one=0;
  ins_param_count=0;
  ins_param_two=0;
  run_core=0;
  }
}
